searchData={"items":[{"type":"module","title":"Phoenix.View","doc":"A module for generating `render/2` functions from templates on disk.\n\nWith design patterns introduced by `Phoenix.LiveView`, this module has fallen\nout of fashion in favor of `Phoenix.Component`, even in non LiveView\napplications. See the \"Replaced by `Phoenix.Component`\" section below.","ref":"Phoenix.View.html"},{"type":"module","title":"Examples - Phoenix.View","doc":"In Phoenix v1.6 and earlier, new Phoenix apps defined a blueprint for views\nat `lib/your_app_web.ex`. It generally looked like this:\n\n    defmodule YourAppWeb do\n      # ...\n\n      def view do\n        quote do\n          use Phoenix.View, root: \"lib/your_app_web/templates\", namespace: YourAppWeb\n\n          # Import convenience functions from controllers\n          import Phoenix.Controller,\n            only: [get_flash: 1, get_flash: 2, view_module: 1, view_template: 1]\n\n          # Use all HTML functionality (forms, tags, etc)\n          use Phoenix.HTML\n\n          import YourAppWeb.ErrorHelpers\n          import YourAppWeb.Gettext\n        end\n      end\n\n      # ...\n    end\n\nThen you could use the definition above to define any view in your application:\n\n    defmodule YourAppWeb.UserView do\n      use YourAppWeb, :view\n    end\n\nBecause we defined the template root to be \"lib/your_app_web/templates\",\n`Phoenix.View` will automatically load all templates at \"your_app_web/templates/user\"\nand include them in the `YourApp.UserView`. For example, imagine we have the\ntemplate:\n\n    # your_app_web/templates/user/index.html.heex\n    Hello  \n\nThe `.heex` extension maps to a template engine which tells Phoenix how\nto compile the code in the file into Elixir source code. After it is\ncompiled, the template can be rendered as:\n\n    Phoenix.View.render_to_string(YourApp.UserView, \"index.html\", name: \"John Doe\")\n    #=> \"Hello John Doe\"","ref":"Phoenix.View.html#module-examples"},{"type":"module","title":"Rendering and formats - Phoenix.View","doc":"`Phoenix.View` renders templates.\n\nA template has a name, which also contains a format. For example,\nin the previous section we have rendered the \"index.html\" template:\n\n    Phoenix.View.render_to_string(YourApp.UserView, \"index.html\", name: \"John Doe\")\n    #=> \"Hello John Doe\"\n\nWhile we got a string at the end, that's not actually what our templates\nrender. Let's take a deeper look:\n\n    Phoenix.View.render(YourApp.UserView, \"index.html\", name: \"John Doe\")\n    #=> ...\n\nThis inner representation allows us to separate how templates render and\nhow they are encoded. For example, if you want to render JSON data, we\ncould do so by adding a \"show.json\" entry to `render/2` in our view:\n\n    defmodule YourAppWeb.UserView do\n      use YourAppWeb, :view\n\n      def render(\"show.json\", %{user: user}) do\n        %{name: user.name, address: user.address}\n      end\n    end\n\nNotice that in order to render JSON data, we don't need to explicitly\nreturn a JSON string! Instead, we just return data that is encodable to\nJSON. Now, when we call:\n\n    Phoenix.View.render_to_string(YourApp.UserView, \"user.json\", user: %User{...})\n\nBecause the template has the `.json` extension, Phoenix knows how to\nencode the map returned for the \"user.json\" template into an actual\nJSON payload to be sent over the wire.\n\nPhoenix ships with some template engines and format encoders, which\ncan be further configured in the Phoenix application. You can read\nmore about format encoders in `Phoenix.Template` documentation.","ref":"Phoenix.View.html#module-rendering-and-formats"},{"type":"module","title":"Replaced by `Phoenix.Component` - Phoenix.View","doc":"In `Phoenix.LiveView`, `Phoenix.View` was replaced by `Phoenix.Component`.\nWith Phoenix v1.7+ we can also use `Phoenix.Component` to render traditional\ntemplates as functional components, using the `embed_templates` function.\n\nFor example, in Phoenix v1.7+, the `YourAppWeb.UserView` above would be\nwritten as:\n\n    defmodule YourAppWeb.UserHTML do\n      use YourAppWeb, :html\n\n      embed_templates \"users/*\"\n    end\n\nThe benefit of `Phoenix.Component` is that it unifies the rendering of\ntraditional request/response life cycles with the composable component\nmodel provided by LiveView.\n\nThe table below summarizes how the defaults changed from Phoenix v1.6 to v1.7:\n\n| Feature                          | Phoenix v1.6                            | Phoenix v1.7                                  |\n| -------------------------------- | --------------------------------------- | --------------------------------------------- |\n| `MyController.action/2` renders  | `MyView.render(\"action.html\", assigns)` | `MyHTML.action(assigns)`                      |\n| Define views at                  | `lib/my_app/views/my_view.ex`           | `lib/my_app/controllers/my_html.ex`           |\n| At the top of your views         | `use MyAppWeb, :view`                   | `use MyAppWeb, :html`                         |\n| Default template language        | `EEx` (`.eex` extension)                | `HEEx` (`.heex` extension)                    |\n| To embed templates from disk     | `use Phoenix.View`                      | `use Phoenix.Component` (+ `embed_templates`) |\n| HTML helpers (forms, links, etc) | `use Phoenix.HTML`                      | `use Phoenix.Component`                       |\n\nHowever, note Phoenix v1.7 is backwards compatible with v1.6 if you want to\nkeep with the old style. The functionality in this module will be maintained\nin the long term though for those who cannot or prefer not to migrate.\n\n#","ref":"Phoenix.View.html#module-replaced-by-phoenix-component"},{"type":"module","title":"Migrating to Phoenix.Component - Phoenix.View","doc":"Migrating your current views to components be done in a few steps. You should\nalso be able to migrate one view at a time.\n\n> It may be helpful to generate a new project using Phoenix v1.7+ to compare\n> code samples during this process.\n\nThe first step is to define `def html` in your `lib/my_app_web.ex` module.\nThis function is similar to `def view`, but it replaces `use Phoenix.View`\nby `use Phoenix.Component` (requires LiveView 0.18.3 or later). We also\nrecomend to add `import Phoenix.View` inside `def html` while migrating.\n\nThen, for each view, you must follow these steps (we will assume the\ncurrent view is called `MyAppWeb.MyView`):\n\n  1. Replace `render_existing/3` calls by `function_exported?/3` checks,\n     according to the `render_existing` documentation.\n\n  2. Replace `use MyApp, :view` by `use MyApp, :html` and invoke\n     `embed_templates \"../templates/my/*\"`. Alternatively, you can move\n     both the HTML file and its templates to the `controllers` directory,\n     to align with Phoenix v1.7 conventions.\n\n  3. Your templates may now break if they are calling `render/2`.\n     You can address this by replacing `render/2` with a function\n     component. For instance, `render(\"_form.html\", changeset: @changeset, user: @user)`\n     must now be called as ` `.\n     If passing all assigns, `render(\"_form.html\", assigns)` becomes\n     ` `\n\n  4. Your templates may now break if they are calling `render_layout/4`.\n     You can address this by converting the layout into a function component\n     that receives its contents as a slot. See `render_layout/4` docs\n\nNow you are using components! Once you convert all views, you should\nbe able to remove `Phoenix.View` as a dependency from your project.\n\nRemove `def view` and also remove the `import Phoenix.View` from\n`def html` in your `lib/my_app_web.ex` module. When doing so,\ncompilation may fail if you are using certain functions:\n\n  * Replace `render/3` with a function component. For instance,\n    `render(OtherView, \"_form.html\", changeset: @changeset, user: @user)`\n    can now be called as ` `.\n    If passing all assigns, `render(OtherView, \"_form.html\", assigns)`\n    becomes ` `.\n\n  * If you are using `Phoenix.View` for APIs, you can remove `Phoenix.View`\n    altogether. Instead of `def render(\"index.html\", assigns)`, use `def users(assigns)`.\n    Instead of `def render(\"show.html\", assigns)`, do `def user(assigns)`.\n    Instead `render_one`/`render_many`, call the `users/1` and `user/1` functions\n    directly.","ref":"Phoenix.View.html#module-migrating-to-phoenix-component"},{"type":"macro","title":"Phoenix.View.__using__/1","doc":"When used, defines the current module as a main view module.","ref":"Phoenix.View.html#__using__/1"},{"type":"macro","title":"Options - Phoenix.View.__using__/1","doc":"* `:root` - the template root to find templates\n  * `:path` - the optional path to search for templates within the `:root`.\n    Defaults to the underscored view module name. A blank string may\n    be provided to use the `:root` path directly as the template lookup path\n  * `:namespace` - the namespace to consider when calculating view paths\n  * `:pattern` - the wildcard pattern to apply to the root\n    when finding templates. Default `\"*\"`\n\nThe `:root` option is required while the `:namespace` defaults to the\nfirst nesting in the module name. For instance, both `MyApp.UserView`\nand `MyApp.Admin.UserView` have namespace `MyApp`.\n\nThe `:namespace` and `:path` options are used to calculate template\nlookup paths. For example, if you are in `MyApp.UserView` and the\nnamespace is `MyApp`, templates are expected at `Path.join(root, \"user\")`.\nOn the other hand, if the view is `MyApp.Admin.UserView`,\nthe path will be `Path.join(root, \"admin/user\")` and so on. For\nexplicit root path locations, the `:path` option can be provided instead.\nThe `:root` and `:path` are joined to form the final lookup path.\nA blank string may be provided to use the `:root` path directly as the\ntemplate lookup path.\n\nSetting the namespace to `MyApp.Admin` in the second example will force\nthe template to also be looked up at `Path.join(root, \"user\")`.","ref":"Phoenix.View.html#__using__/1-options"},{"type":"function","title":"Phoenix.View.module_to_template_root/3","doc":"Converts a module, without the suffix, to a template root.","ref":"Phoenix.View.html#module_to_template_root/3"},{"type":"function","title":"Examples - Phoenix.View.module_to_template_root/3","doc":"iex> Phoenix.View.module_to_template_root(MyApp.UserView, MyApp, \"View\")\n    \"user\"\n\n    iex> Phoenix.View.module_to_template_root(MyApp.Admin.User, MyApp, \"View\")\n    \"admin/user\"\n\n    iex> Phoenix.View.module_to_template_root(MyApp.Admin.User, MyApp.Admin, \"View\")\n    \"user\"\n\n    iex> Phoenix.View.module_to_template_root(MyApp.View, MyApp, \"View\")\n    \"\"\n\n    iex> Phoenix.View.module_to_template_root(MyApp.View, MyApp.View, \"View\")\n    \"\"","ref":"Phoenix.View.html#module_to_template_root/3-examples"},{"type":"function","title":"Phoenix.View.render/3","doc":"Renders a template.\n\nIt expects the view module, the template as a string, and a\nset of assigns.\n\nNotice that this function returns the inner representation of a\ntemplate. If you want the encoded template as a result, use\n`render_to_iodata/3` instead.","ref":"Phoenix.View.html#render/3"},{"type":"function","title":"Examples - Phoenix.View.render/3","doc":"Phoenix.View.render(YourApp.UserView, \"index.html\", name: \"John Doe\")\n    #=> {:safe, \"Hello John Doe\"}","ref":"Phoenix.View.html#render/3-examples"},{"type":"function","title":"Assigns - Phoenix.View.render/3","doc":"Assigns are meant to be user data that will be available in templates.\nHowever, there are keys under assigns that are specially handled by\nPhoenix, they are:\n\n  * `:layout` - tells Phoenix to wrap the rendered result in the\n    given layout. See next section","ref":"Phoenix.View.html#render/3-assigns"},{"type":"function","title":"Layouts - Phoenix.View.render/3","doc":"Templates can be rendered within other templates using the `:layout`\noption. `:layout` accepts a tuple of the form\n`{LayoutModule, \"template.extension\"}`.\n\nTo template that goes inside the layout will be placed in the `@inner_content`\nassign:","ref":"Phoenix.View.html#render/3-layouts"},{"type":"function","title":"Phoenix.View.render_existing/3","doc":"Renders a template only if it exists.\n\n> Note: Using this functionality has been discouraged in\n> recent Phoenix versions, see the \"Alternatives\" section\n> below.\n\nThis function works the same as `render/3`, but returns\n`nil` instead of raising. This is often used with\n`Phoenix.Controller.view_module/1` and `Phoenix.Controller.view_template/1`,\nwhich must be imported into your views. See the \"Examples\"\nsection below.","ref":"Phoenix.View.html#render_existing/3"},{"type":"function","title":"Alternatives - Phoenix.View.render_existing/3","doc":"This function is discouraged. If you need to render something\nconditionally, the simplest way is to check for an optional\nfunction in your views.\n\nConsider the case where the application has a sidebar in its\nlayout and it wants certain views to render additional buttons\nin the sidebar. Inside your sidebar, you could do:\n\n     \n       \n         \n       \n     \n\nIf you are using Phoenix.LiveView, you could do similar by\naccessing the view under `@socket`:\n\n     \n       \n         \n       \n     \n\nThen, in your view or live view, you do:\n\n    def sidebar_additions(assigns) do\n      ~H\\\"\"\"\n      ...my additional buttons...\n      \\\"\"\"","ref":"Phoenix.View.html#render_existing/3-alternatives"},{"type":"function","title":"Using render_existing - Phoenix.View.render_existing/3","doc":"Consider the case where the application wants to allow entries\nto be added to a sidebar. This feature could be achieved with:\n\n     \n\nThen the module under `view_module(@conn)` can decide to provide\nscripts with either a precompiled template, or by implementing the\nfunction directly, ie:\n\n    def render(\"sidebar_additions.html\", _assigns) do\n      ~H\"\"\"\n      ...my additional buttons...\n      \"\"\"\n    end\n\nTo use a precompiled template, create a `scripts.html.eex` file in\nthe `templates` directory for the corresponding view you want it to\nrender for. For example, for the `UserView`, create the `scripts.html.eex`\nfile at `your_app_web/templates/user/`.","ref":"Phoenix.View.html#render_existing/3-using-render_existing"},{"type":"function","title":"Phoenix.View.render_layout/4","doc":"Renders the given layout passing the given `do/end` block\nas `@inner_content`.\n\nThis can be useful to implement nested layouts. For example,\nimagine you have an application layout like this:\n\n    # layout/app.html.heex\n     \n     \n       Title \n     \n     \n       ... \n       \n     \n\nThis layout is used by many parts of your application. However,\nthere is a subsection of your application that wants to also add\na sidebar. Let's call it \"blog.html\". You can build on top of the\nexisting layout in two steps. First, define the blog layout:\n\n    # layout/blog.html.heex\n     \n       ... \n       \n     \n\nAnd now you can simply use it from your controller:\n\n    plug :put_layout, \"blog.html\"","ref":"Phoenix.View.html#render_layout/4"},{"type":"function","title":"Alternatives - Phoenix.View.render_layout/4","doc":"`render_layout/4` is discouraged in favor of components.\nIf you need to share functionality, you can create components\nwith bits of functionality you want to reuse. For example,\nthe code above could be rewritten with a layout component:\n\n    def layout(assigns) do\n      ~H\"\"\"\n       \n         \n         \n       \n      \"\"\"\n    end\n\nWhich can be used as:\n\n     \n      Main content\n     \n\nOr:\n\n     \n       Additional sidebar content \n      Main content\n     \n\nThe advantage of using components is that you can handle all\nof the sidebar markup inside the parent layout component,\ninstead of spreading it across multiple files.","ref":"Phoenix.View.html#render_layout/4-alternatives"},{"type":"function","title":"Phoenix.View.render_many/4","doc":"Renders a collection.\n\nIt receives a collection as an enumerable of structs and returns\nthe rendered collection in a list. This is typically used to render\na collection as structured data. For example, to render a list of\nusers to json:\n\n    render_many(users, UserView, \"show.json\")\n\nwhich is roughly equivalent to:\n\n    Enum.map(users, fn user ->\n      render(UserView, \"show.json\", user: user)\n    end)\n\nThe underlying user is passed to the view and template as `:user`,\nwhich is inferred from the view name. The name of the key\nin assigns can be customized with the `:as` option:\n\n    render_many(users, UserView, \"show.json\", as: :data)\n\nis roughly equivalent to:\n\n    Enum.map(users, fn user ->\n      render(UserView, \"show.json\", data: user)\n    end)","ref":"Phoenix.View.html#render_many/4"},{"type":"function","title":"Phoenix.View.render_one/4","doc":"Renders a single item if not nil.\n\nThe following:\n\n    render_one(user, UserView, \"show.json\")\n\nis roughly equivalent to:\n\n    if user != nil do\n      render(UserView, \"show.json\", user: user)\n    end\n\nThe underlying user is passed to the view and template as\n`:user`, which is inflected from the view name. The name\nof the key in assigns can be customized with the `:as` option:\n\n    render_one(user, UserView, \"show.json\", as: :data)\n\nis roughly equivalent to:\n\n    if user != nil do\n      render(UserView, \"show.json\", data: user)\n    end","ref":"Phoenix.View.html#render_one/4"},{"type":"function","title":"Phoenix.View.render_to_iodata/3","doc":"Renders the template and returns iodata.","ref":"Phoenix.View.html#render_to_iodata/3"},{"type":"function","title":"Phoenix.View.render_to_string/3","doc":"Renders the template and returns a string.","ref":"Phoenix.View.html#render_to_string/3"},{"type":"function","title":"Phoenix.View.template_path_to_name/2","doc":"Converts the template path into the template name.","ref":"Phoenix.View.html#template_path_to_name/2"},{"type":"function","title":"Examples - Phoenix.View.template_path_to_name/2","doc":"iex> Phoenix.View.template_path_to_name(\n    ...>   \"lib/templates/admin/users/show.html.eex\",\n    ...>   \"lib/templates\"\n    ...> )\n    \"admin/users/show.html\"","ref":"Phoenix.View.html#template_path_to_name/2-examples"},{"type":"exception","title":"Phoenix.Template.UndefinedError","doc":"Exception raised when a template cannot be found.","ref":"Phoenix.Template.UndefinedError.html"},{"type":"function","title":"Phoenix.Template.UndefinedError.message/1","doc":"","ref":"Phoenix.Template.UndefinedError.html#message/1"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"2.0.3 (2023-10-28) - CHANGELOG","doc":"* Relax Phoenix.HTML dependency","ref":"changelog.html#2-0-3-2023-10-28"},{"type":"extras","title":"2.0.2 (2022-11-07) - CHANGELOG","doc":"* Fix regression where directories were not being tracked for new files","ref":"changelog.html#2-0-2-2022-11-07"},{"type":"extras","title":"2.0.1 (2022-10-29) - CHANGELOG","doc":"* Undeprecate `Phoenix.View.render_layout/4`","ref":"changelog.html#2-0-1-2022-10-29"},{"type":"extras","title":"2.0.0 (2022-10-26) - CHANGELOG","doc":"* Extract `Phoenix.Template` to a separete dependency: `phoenix_template`\n  * Document replacing `Phoenix.View` with `Phoenix.Component`\n  * Deprecate `Phoenix.View.render_layout/4` in favor of `Phoenix.Component` with slots instead","ref":"changelog.html#2-0-0-2022-10-26"},{"type":"extras","title":"1.1.2 (2022-02-02) - CHANGELOG","doc":"* Fix dialyzer warnings on `template_not_found`","ref":"changelog.html#1-1-2-2022-02-02"},{"type":"extras","title":"1.1.1 (2022-01-31) - CHANGELOG","doc":"* Add compile-time dependencies to template engines","ref":"changelog.html#1-1-1-2022-01-31"},{"type":"extras","title":"1.1.0 (2022-01-06) - CHANGELOG","doc":"* Do not add compile time dependencies on arguments given to Phoenix.View and Phoenix.Template\n  * Soft-deprecate `render_existing/3` in favor of `function_exported?/3` checks","ref":"changelog.html#1-1-0-2022-01-06"},{"type":"extras","title":"1.0.0 (2021-07-18) - CHANGELOG","doc":"* Initial release","ref":"changelog.html#1-0-0-2021-07-18"}],"content_type":"text/markdown"}